//==============================================================================
//	File:		$URL: 
//	Copyright:	Copyright 2005-2008 UC Berkeley
//==============================================================================

//==============================================================================
//	Section:	License
//==============================================================================
//	Copyright (c) 2005-2008, Regents of the University of California
//	All rights reserved.
//
//	Redistribution and use in source and binary forms, with or without modification,
//	are permitted provided that the following conditions are met:
//
//		- Redistributions of source code must retain the above copyright notice,
//			this list of conditions and the following disclaimer.
//		- Redistributions in binary form must reproduce the above copyright
//			notice, this list of conditions and the following disclaimer
//			in the documentation and/or other materials provided with the
//			distribution.
//		- Neither the name of the University of California, Berkeley nor the
//			names of its contributors may be used to endorse or promote
//			products derived from this software without specific prior
//			written permission.
//
//	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
//	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
//	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//	DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
//	ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
//	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
//	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
//	ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
//	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//==============================================================================

//------------------------------------------------------------------------------
//	Section:	Includes
//------------------------------------------------------------------------------
`include "Const.v"
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//	Module:		FPGA_TOP_ML505 video template
//	Desc:		Top level interface from a Xilinx XC5VLX50T or XC5VLX110T FPGA
//				to the ML505-50 or ML505-110 Board respectively.
//	Author:		Brandon Myers
//	Version:	$Revision: 17012 $
//------------------------------------------------------------------------------
module FPGA_TOP_ML505(
			//------------------------------------------------------------------
			//	Compass Switches & LEDs
			//------------------------------------------------------------------
			GPIO_COMPLED,	// OUT(5b), W4, S3, N2, E1, C0
			GPIO_COMPSW,	// IN(5b), W4, S3, N2, E1, C0
			//------------------------------------------------------------------
	
			//------------------------------------------------------------------
			//	 General IO (Dipswitch, LED, Rotary Encoder & Bus Error LEDs)
			//------------------------------------------------------------------
			GPIO_DIP,
			GPIO_LED,
			//------------------------------------------------------------------
	
			//------------------------------------------------------------------
			//	Clocks
			//------------------------------------------------------------------
			CLK_33MHZ_FPGA,
			USER_CLK_100MHZ,
			//------------------------------------------------------------------
	
			//------------------------------------------------------------------
			//	Serial Ports
			//------------------------------------------------------------------
			FPGA_SERIAL_RX,
			FPGA_SERIAL_TX,
			//------------------------------------------------------------------
	
			//------------------------------------------------------------------
			//	Audio (AC97 & Piezo)
			//------------------------------------------------------------------
			//	Note that the reset is in with the FLASH memory signals (shared)
			//------------------------------------------------------------------
	
			//------------------------------------------------------------------
			//	DRAM (DDR2 SODIMM)
			//------------------------------------------------------------------
			//------------------------------------------------------------------
	
			//------------------------------------------------------------------
			//	PS/2 Ports
			//------------------------------------------------------------------
			//------------------------------------------------------------------

			//------------------------------------------------------------------
			//	LCD (16x2)
			//------------------------------------------------------------------
			//------------------------------------------------------------------
		
			//------------------------------------------------------------------
			//	DVI Video
			//------------------------------------------------------------------
			DVI_D,

			DVI_DE,
			DVI_GPIO1,
			DVI_H,
			DVI_RESET_B,
			DVI_V,
			DVI_XCLK_N,
			DVI_XCLK_P,

			I2C_SCL_DVI,
			I2C_SDA_DVI,
			//------------------------------------------------------------------
	
			//------------------------------------------------------------------
			//	FPGA Configuration & Management
			//------------------------------------------------------------------
			//------------------------------------------------------------------
		
			//------------------------------------------------------------------
			//	Headers
			//------------------------------------------------------------------
			//------------------------------------------------------------------

			//------------------------------------------------------------------
			//	Main I2C: EEPROM, FAN, XGI
			//------------------------------------------------------------------
			//------------------------------------------------------------------

			//------------------------------------------------------------------
			//	Bank 118 GTPS: PCIe & Loopback
			//------------------------------------------------------------------
			//------------------------------------------------------------------

			//------------------------------------------------------------------
			//	Ethernet PHY (non-SGMII)
			//------------------------------------------------------------------
			PHY_COL,
			PHY_CRS,
			PHY_INT,
			PHY_MDC,
			PHY_MDIO,
			PHY_RESET,
			
			PHY_RXCLK,
			PHY_RXCTL_RXDV,
			PHY_RXER,
			PHY_RXD,
			
			PHY_TXC_GTXCLK,
			PHY_TXCLK,
			PHY_TXCTL_TXEN,
			PHY_TXER,
			PHY_TXD,
			//------------------------------------------------------------------

			//------------------------------------------------------------------
			//	Bank 116 GTPs: Small Form-Factor Pluggable & SMA
			//------------------------------------------------------------------
			//------------------------------------------------------------------
	
			//------------------------------------------------------------------
			//	Bank 112 GTPs: SGMII & Loopback
			//------------------------------------------------------------------
			//------------------------------------------------------------------

			//------------------------------------------------------------------
			//	Bank 114 GTPs: SATA Connectors
			//------------------------------------------------------------------
			//------------------------------------------------------------------

			//------------------------------------------------------------------
			//	ZBT SRAM & Linear Flash Memory
			//------------------------------------------------------------------
			SRAM_CLK,
			SRAM_CLK_FB,
			SRAM_CS_B,
			SRAM_ADV_LD_B,
			
			SRAM_FLASH_WE_B,
			SRAM_MODE,
			SRAM_OE_B,
			
			SRAM_BW,

			SRAM_DQP,

			SRAM_FLASH_A,
			SRAM_FLASH_D,
			SRAM_D,
			//------------------------------------------------------------------

			//------------------------------------------------------------------
			//	System ACE & Cypress USB
			//------------------------------------------------------------------
			//------------------------------------------------------------------

			//------------------------------------------------------------------
			//	Tracing
			//------------------------------------------------------------------
			//------------------------------------------------------------------


			//------------------------------------------------------------------
			//	VGA (Input)
			//------------------------------------------------------------------
			//------------------------------------------------------------------
		); /* synthesis syn_noprune=1 */
	//--------------------------------------------------------------------------
	//	Compass Switches & LEDs (W4, S3, N2, E1, C0)
	//--------------------------------------------------------------------------
	output	[4:0]			GPIO_COMPLED 			/* synthesis xc_loc = "AF23,AG12,AF13,AG23,E8" xc_padtype = "OBUF_LVCMOS33_S_2" xc_pulldown = 1 chip_pin = "AF23,AG12,AF13,AG23,E8" altera_attribute="-name IO_STANDARD LVCMOS33_S_2" */;
	// synthesis attribute LOC of					GPIO_COMPLED is			"AF23,AG12,AF13,AG23,E8"
	// synthesis attribute IOSTANDARD of			GPIO_COMPLED is			"LVCMOS33"
	// synthesis attribute SLEW of					GPIO_COMPLED is			"SLOW"
	// synthesis attribute DRIVE of					GPIO_COMPLED is			"2"
	input	[4:0]			GPIO_COMPSW				/* synthesis xc_loc = "AJ7,V8,U8,AK7,AJ6" xc_padtype = "IBUF_LVCMOS33" xc_pulldown = 1 chip_pin = "AJ7,V8,U8,AK7,AJ6" altera_attribute="-name IO_STANDARD LVCMOS33" */;
	// synthesis attribute LOC of					GPIO_COMPSW is			"AJ7,V8,U8,AK7,AJ6"
	// synthesis attribute IOSTANDARD of			GPIO_COMPSW is			"LVCMOS33"
	
	genvar			i;
	`ifdef PULL
		generate for (i = 0; i < 5; i = i + 1) begin:PULL_GPIO_COMPXXX
			pulldown(GPIO_COMPLED[i]);
			pulldown(GPIO_COMPSW[i]);
		end endgenerate
	`endif
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	 General IO (Dipswitch, LED, Rotary Encoder & Bus Error LEDs)
	//--------------------------------------------------------------------------
	input	[7:0]			GPIO_DIP				/* synthesis xc_loc = "AC24,AC25,AE26,AE27,AF26,AF25,AG27,U25" xc_padtype = "IBUF_LVCMOS25" chip_pin = "AC24,AC25,AE26,AE27,AF26,AF25,AG27,U25" altera_attribute="-name IO_STANDARD LVCMOS25" */;
	// synthesis attribute LOC of					GPIO_DIP is				"AC24,AC25,AE26,AE27,AF26,AF25,AG27,U25"
	// synthesis attribute IOSTANDARD of			GPIO_DIP is				"LVCMOS25"
	output	[7:0]			GPIO_LED				/* synthesis xc_loc = "AE24,AD24,AD25,G16,AD26,G15,L18,H18" xc_padtype = "OBUF_LVCMOS25_S_2" xc_pulldown = 1 chip_pin = "AE24,AD24,AD25,G16,AD26,G15,L18,H18" altera_attribute="-name IO_STANDARD LVCMOS25_S_2" */;
	// synthesis attribute LOC of					GPIO_LED is				"AE24,AD24,AD25,G16,AD26,G15,L18,H18"
	// synthesis attribute IOSTANDARD of			GPIO_LED is				"LVCMOS25"
	// synthesis attribute SLEW of					GPIO_LED is				"SLOW"
	// synthesis attribute DRIVE of					GPIO_LED is				"2"
	
	`ifdef PULL
		generate for (i = 0; i < 8; i = i + 1) begin:PULL_GPIO_XXX
			pulldown(GPIO_DIP[i]);
			pulldown(GPIO_LED[i]);
		end endgenerate
	`endif
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	Clocks
	//--------------------------------------------------------------------------
	input					CLK_33MHZ_FPGA;
	input					USER_CLK_100MHZ;
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	Serial Ports
	//--------------------------------------------------------------------------
	input	[2:1]			FPGA_SERIAL_RX;
	output	[2:1]			FPGA_SERIAL_TX;
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	Audio (AC97 & Piezo)
	//--------------------------------------------------------------------------
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	DRAM (DDR2 SODIMM)
	//--------------------------------------------------------------------------
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	PS/2 Ports
	//--------------------------------------------------------------------------
	//--------------------------------------------------------------------------

	//--------------------------------------------------------------------------
	//	LCD (16x2)
	//--------------------------------------------------------------------------
	//--------------------------------------------------------------------------
		
	//--------------------------------------------------------------------------
	//	DVI Video
	//--------------------------------------------------------------------------
	output	[11:0]			DVI_D;

	output					DVI_DE;
	output					DVI_GPIO1;"
	output					DVI_H;
	output					DVI_RESET_B;
	output					DVI_V;
	output					DVI_XCLK_N;
	output					DVI_XCLK_P;

	inout					I2C_SCL_DVI				/* synthesis xc_loc = "U27" chip_pin = "U27" */;
	// synthesis attribute LOC of					I2C_SCL_DVI is			"U27"
	inout					I2C_SDA_DVI				/* synthesis xc_loc = "T29" chip_pin = "T29" */;
	// synthesis attribute LOC of					I2C_SDA_DVI is			"T29"
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	FPGA Configuration & Management
	//--------------------------------------------------------------------------
	//--------------------------------------------------------------------------
		
	//--------------------------------------------------------------------------
	//	Headers
	//--------------------------------------------------------------------------
	//--------------------------------------------------------------------------

	//--------------------------------------------------------------------------
	//	Main I2C: EEPROM, FAN, XGI
	//--------------------------------------------------------------------------
	//--------------------------------------------------------------------------

	//--------------------------------------------------------------------------
	//	Bank 118 GTPS: PCIe & Loopback
	//--------------------------------------------------------------------------
	//--------------------------------------------------------------------------

	//--------------------------------------------------------------------------
	//	Ethernet PHY (non-SGMII)
	//--------------------------------------------------------------------------
	input					PHY_COL;
	input					PHY_CRS;
	input					PHY_INT_;
	output					PHY_MDC;
	inout					PHY_MDIO;
	output					PHY_RESET_;
	
	input					PHY_RXCLK;
	// synthesis attribute PERIOD of				PHY_RXCLK is			"125 MHz"
	
	input					PHY_RXCTL_RXDV;
	input					PHY_RXER;
	input	[7:0]			PHY_RXD;

	output					PHY_TXC_GTXCLK;
	// synthesis attribute PERIOD of				PHY_TXC_GTXCLK is		"125 MHz"
	
	input					PHY_TXCLK;
	// synthesis attribute PERIOD of				PHY_TXCLK is			"25 MHz"
	
	output					PHY_TXCTL_TXEN;
	output					PHY_TXER;
	output	[7:0]			PHY_TXD;
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	Bank 116 GTPs: Small Form-Factor Pluggable & SMA
	//--------------------------------------------------------------------------
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	Bank 112 GTPs: SGMII & Loopback
	//--------------------------------------------------------------------------
	//--------------------------------------------------------------------------

	//--------------------------------------------------------------------------
	//	Bank 114 GTPs: SATA Connectors
	//--------------------------------------------------------------------------
	//--------------------------------------------------------------------------

	//--------------------------------------------------------------------------
	//	ZBT SRAM & Linear Flash Memory
	//--------------------------------------------------------------------------
	output					SRAM_CLK;			
	input					SRAM_CLK_FB;				
	
	output					SRAM_CS_B;
	
	output					SRAM_ADV_LD_B;
	
	output					SRAM_FLASH_WE_B;
	
	output					SRAM_MODE;
	
	output					SRAM_OE_B;
	output	[3:0]			SRAM_BW;
	

	inout	[3:0]			SRAM_DQP;
	

	output	[21:0]			SRAM_FLASH_A;
	
	inout	[15:0]			SRAM_FLASH_D;

	inout	[31:16]			SRAM_D;
	
	
	`ifdef PULL
		generate for (i = 0; i < 32; i = i + 1) begin:SRAM_PULLDOWN
			pulldown(SRAM_D[i]);
		end endgenerate
	`endif
	//--------------------------------------------------------------------------

	//--------------------------------------------------------------------------
	//	System ACE & Cypress USB
	//--------------------------------------------------------------------------
	//--------------------------------------------------------------------------

	//--------------------------------------------------------------------------
	//	Tracing
	//--------------------------------------------------------------------------
	//--------------------------------------------------------------------------

	//--------------------------------------------------------------------------
	//	VGA (Input)
	//--------------------------------------------------------------------------
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	Parameters
	//--------------------------------------------------------------------------
	parameter	VideoClockFreq  =	   50000000,
				FRClockFreq		=		100000000;
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	// Wires
	//--------------------------------------------------------------------------
	wire FRClock;  // clock for framebuffer and sram
	wire VideoClock; // clock for Video
	wire Clock100_buf;
	
	wire 		VideoValid;
	wire 		VideoReady;
	wire [23:0] Pixel;
	//--------------------------------------------------------------------------
	 
	//--------------------------------------------------------------------------
	//	Clock & Reset
	//--------------------------------------------------------------------------  
	VideoClockGen 
		clockgen (	.CLKIN_IN(USER_CLK_100MHZ),
					.RST_IN(/* either 1'b0 or a reset generated using Clock100_buf*/),
					.CLKDV_OUT(VideoClock), 
					.CLKIN_IBUFG_OUT(Clock100_buf), 
					.CLK0_OUT(FRClock));
	//--------------------------------------------------------------------------
	
	
	//--------------------------------------------------------------------------
	// WriteFIFOs
	//--------------------------------------------------------------------------
	
	// this module is optional since it interfaces only with your own modules.  You
	// can use your own fifos if you want.
	
	WriteFIFOs wrfifos (
		.CPUClock(		/* CPU Clock */),
		.FRClock(		FRClock),
		.AsyncReset(	/* fill in reset */),

		.CPUDataOut(		), // Pixel color to write, coming out of FIFO.
		.CPUCoordOut(		), // Pixel location to write, coming out of FIFO.
		.CPUValidRequest( ), // There is valid data available
		.CPUTake(			), // The CPU write request was chosen and should be dequeued.
		
		.CPUCoordIn(		), // Pixel coordinate address to be written to Framebuffer.
		.CPUDataIn(			), // Pixel color to be written to Framebuffer.
		.CPUWriteEnable(	), // 'MemWrite' signal to Framebuffer.
		.CPUWriteReady(	), // Ready to accept CPU writes (room in FIFO).
	
		.LEDataOut(), 
		.LECoordOut(), 
		.LEValidRequest(),
		.LETake(), 
		.LECoordIn(), 
		.LEDataIn(), 
		.LEWriteEnable(),
		.LEWriteReady() 
	);
	//--------------------------------------------------------------------------
	
	
	//--------------------------------------------------------------------------
	// SRAM Controller
	//--------------------------------------------------------------------------
	SRAMController sramcont (
		.Clock(			FRClock),
		.Reset(			/*fill in reset*/),
	
		// user interface
		/* add interface ports */
		
		// to device
		.SRAM_CLK(		SRAM_CLK),  // should assign SRAM_CLK to same as FRClock somewhere
		.SRAM_CS_B(		SRAM_CS_B),
		.SRAM_ADV_LD_B(SRAM_ADV_LD_B),
		.SRAM_WE_B(		SRAM_FLASH_WE_B),
		.SRAM_MODE(		SRAM_MODE),
		.SRAM_OE_B(		SRAM_OE_B),
		.SRAM_BW(		SRAM_BW),
		.SRAM_DQP(		SRAM_DQP),
		.SRAM_A(		SRAM_FLASH_A),
		.SRAM_Data(		{SRAM_D[31:16], SRAM_FLASH_D[15:0]}));
  
	//--------------------------------------------------------------------------
  
	//-------------------------------------------------------------------------
	// Pixel interface and DVI
	//-------------------------------------------------------------------------
	Video video (
		.Reset(/* fill in reset */),
		.PixelsValid(/* fill in */),
		.Pixel0In(/* fill in */),   //even pixel
		.Pixel1In(/* fill in */),  //odd pixel
		.Address(/* fill in */),
		.AddressValid(/* fill in */),
		.FrameBufferClkIn(FRClock),
		.VideoClkIn(VideoClock),
	
		.VideoOut(Pixel),
		.VideoReady(VideoReady),
		.VideoValid(VideoValid)
	);
	//-------------------------------------------------------------------------
  
	//-------------------------------------------------------------------------
	// DVI
	//-------------------------------------------------------------------------
	// Resolution				Width	FrontH	PulseH	BackH	Height	FrontV	PulseV	BackV	ClockFreq
	//	VESA 800x600,72Hz:	 1040	56		   120		64		666		37		   6		   23		50000000
	// SVGA 800x600,75Hz:    1056 16       80       160   625      1        3        21    49500000
	DVI 			#(			.ClockFreq(		VideoClockFreq),
								.Width(			1040),
								.FrontH(			56),
								.PulseH(			120),
								.BackH(			64),
								.Height(			666),
								.FrontV(			37),
								.PulseV(			6),
								.BackV(			23))
					dvi(		.Clock(			VideoClock),
								.Reset(			/* fill in reset */),
								.DVI_D(			DVI_D),
								.DVI_DE(			DVI_DE),
								.DVI_H(			DVI_H),
								.DVI_V(			DVI_V),
								.DVI_RESET_B(	DVI_RESET_B),
								.DVI_XCLK_N(	DVI_XCLK_N),
								.DVI_XCLK_P(	DVI_XCLK_P),
								.I2C_SCL_DVI(	I2C_SCL_DVI),
								.I2C_SDA_DVI(	I2C_SDA_DVI),
								.Video(			Pixel),
								.VideoReady( 	VideoReady),
								.VideoValid( 	VideoValid));
	//-------------------------------------------------------------------------
	
endmodule
//-------------------------------------------------------------------------