`timescale 1ns / 1ps

module SequentialAddressGenerator(Clock, Reset, Next, AddressOut, AddressValid);
	
	parameter PixelsPerRead = 2;
	localparam NumAddresses = (800*600)/PixelsPerRead;
	parameter AWidth = 18;// ceil log2 NumAddresses
	
	parameter CyclesPerValid = 4;  // 100MHz/50MHz/PixelsPerRead
	parameter ValWidth = 2;  // log2 CyclesPerValid
	
	input Clock, Reset, Next;
	output	[AWidth-1:0]	AddressOut;
	output						AddressValid;
	
	wire [AWidth+ValWidth-1:0] CountOut;
	wire Limit;
	
	Counter #(.Width(AWidth+ValWidth)) 
		cnt 	(	.Clock(	Clock), 
						.Reset(	Reset|Limit), 
						.Set(		1'b0),
						.Load(	1'b0), 
						.Enable(	Next), 
						.In(		{(AWidth+ValWidth){1'bx}}), 
						.Count(	CountOut));
					
	CountCompare #(.Width(AWidth+ValWidth), .Compare({NumAddresses-1, 2'b11}))
			ccmp	(	.Count(CountOut),
						.TerminalCount(Limit));
					
	assign AddressValid = (~|CountOut[ValWidth-1:0])&Next;
	
	assign AddressOut = CountOut[AWidth+ValWidth-1:ValWidth];

endmodule
