module WriteFIFOs (
	CPUClock,  // CPU and line engine Clock
	FRClock,    // Framebuffer Clock (arbiter and sram ctrl)
	AsyncReset,

	CPUDataOut, // Pixel color to write, coming out of FIFO.
	CPUCoordOut, // Pixel location to write, coming out of FIFO.
	CPUValidRequest, // There is valid data available
	CPUTake, // The CPU write request was chosen and should be dequeued.
	CPUCoordIn, // Pixel coordinate address to be written to Framebuffer.
	CPUDataIn, // Pixel color to be written to Framebuffer.
	CPUWriteEnable, // MemWrite signal to Framebuffer.
	CPUWriteReady, // Ready to accept CPU writes (room in FIFO).
	
	LEDataOut, 
	LECoordOut, 
	LEValidRequest,
	LETake, 
	LECoordIn, 
	LEDataIn, 
	LEWriteEnable,
	LEWriteReady 
);

	input CPUClock;
	input FRClock;
	input AsyncReset;

	output [15:0] CPUDataOut; // Pixel color to write, coming out of FIFO.
	output [19:0] CPUCoordOut; // Pixel location to write, coming out of FIFO.
	output 		  CPUValidRequest; // There is valid data available
	input 		  CPUTake; // The CPU write request was chosen and should be dequeued.
	input  [19:0] CPUCoordIn; // Pixel coordinate address to be written to Framebuffer.
	input  [15:0] CPUDataIn; // Pixel color to be written to Framebuffer.
	input 		  CPUWriteEnable; // MemWrite signal to Framebuffer.
	output 		  CPUWriteReady; // Ready to accept CPU writes (room in FIFO).
	
	output [15:0] LEDataOut;
	output [19:0] LECoordOut;
	output 		  LEValidRequest;
	input 		  LETake;
	input  [19:0] LECoordIn; 
	input  [15:0] LEDataIn;
	input         LEWriteEnable;
	output        LEWriteReady; 
	
	
	wire cf_full, cf_almost_full, cf_empty;
	wire [9:0] c_data_count; //useful for simulation
	FBWriteFIFO cpufifo (
	.din({CPUDataIn, CPUCoordIn}), // Bus [35 : 0] 
	.rd_clk(FRClock),
	.rd_en(CPUTake),
	.rst(AsyncReset),
	.wr_clk(CPUClock),
	.wr_en(CPUWriteEnable),
	.almost_full(cf_almost_full),
	.dout({CPUDataOut, CPUCoordOut}), // Bus [35 : 0] 
	.empty(cf_empty),
	.full(cf_full),
	.rd_data_count(c_data_count)); // Bus [9 : 0] 
	
	assign CPUWriteReady = ~(cf_almost_full|cf_full);
	assign CPUValidRequest = ~cf_empty;
	
	wire le_full, le_almost_full, le_empty;
	wire [9:0] le_data_count; //useful for simulation
	FBWriteFIFO lefifo (
	.din({LEDataIn, LECoordIn}), // Bus [35 : 0] 
	.rd_clk(FRClock),
	.rd_en(LETake),
	.rst(AsyncReset),
	.wr_clk(CPUClock),
	.wr_en(LEWriteEnable),
	.almost_full(le_almost_full),
	.dout({LEDataOut, LECoordOut}), // Bus [35 : 0] 
	.empty(le_empty),
	.full(le_full),
	.rd_data_count(le_data_count)); // Bus [9 : 0] 

	assign LEWriteReady = ~(le_almost_full|le_full);
	assign LEValidRequest = ~le_empty;

endmodule
